@@ -10,8 +10,10 @@
 
 #include <QQmlEngine>
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <private/qv4arraybuffer_p.h>
 #include <private/qv4engine_p.h>
+#endif
 
 using namespace KItinerary;
 
@@ -29,5 +31,9 @@ bool BinaryDocumentProcessor::matches(const ExtractorFilter &filter, const Extra
 
 QJSValue BinaryDocumentProcessor::contentToScriptValue(const ExtractorDocumentNode &node, QJSEngine *engine) const
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     return QJSValue(engine->handle(), engine->handle()->newArrayBuffer(node.content<QByteArray>())->asReturnedValue());
+#else
+    return QJSValue(QJSManagedValue(QVariant(node.content<QByteArray>()), engine));
+#endif
 }
