@@ -15,6 +15,8 @@
 
 #include <QDateTime>
 #include <QDebug>
+#include <QJsonDocument>
+#include <QJsonObject>
 
 #include <zlib.h>
 
@@ -123,7 +125,42 @@ bool Uic9183Parser::isValid() const
 
 QString Uic9183Parser::pnr() const
 {
-    return findBlock<Uic9183Head>().ticketKey().left(6);
+    const auto head = findBlock<Uic9183Head>();
+    const auto key = head.ticketKey().trimmed();
+    const auto issuerId = head.issuerCompanyCodeNumeric();
+
+    // try to make this match what's printed on the matching tickets...
+    if (issuerId == 80 && (key.size() == 8 || key.size() == 9) && key.at(6) == QLatin1Char('-') && key.at(7).isDigit()) {
+        return key.left(6); // DB domestic
+    }
+    if (issuerId == 80 && key.size() == 13 && key.endsWith(QLatin1String("0101"))) {
+        return key.left(9); // DB domestic part of an international order
+    }
+    if (issuerId == 1184 && key.size() == 9 && key.at(7) == QLatin1Char('_') && key.at(8).isDigit()) {
+        return key.left(7); // NS
+    }
+
+    return key;
+}
+
+QString Uic9183Parser::name() const
+{
+    // DB vendor block
+    const auto b = findBlock<Vendor0080BLBlock>();
+    if (b.isValid()) {
+        const auto sblock = b.findSubBlock("001");
+        if (!sblock.isNull()) {
+            return QString::fromUtf8(sblock.content(), sblock.contentSize());
+        }
+    }
+
+    // RCT2
+    const auto rct2 = rct2Ticket();
+    if (rct2.isValid()) {
+        return rct2.title();
+    }
+
+    return {};
 }
 
 QString Uic9183Parser::carrierId() const
@@ -131,6 +168,60 @@ QString Uic9183Parser::carrierId() const
     return findBlock<Uic9183Head>().issuerCompanyCodeString();
 }
 
+QDateTime Uic9183Parser::validFrom() const
+{
+    // DB vendor block
+    if (const auto b = findBlock<Vendor0080BLBlock>(); b.isValid() && b.orderBlockCount() == 1) {
+        return QDateTime(b.orderBlock(0).validFrom(), {0, 0, 0});
+    }
+
+    // ÖBB vender block
+    if (const auto b = findBlock("118199"); !b.isNull()) {
+        const auto obj = QJsonDocument::fromJson(QByteArray::fromRawData(b.content(), b.contentSize())).object();
+        auto dt = QDateTime::fromString(obj.value(QLatin1String("V")).toString(), QStringLiteral("yyMMddhhmm"));
+        if (dt.date().year() < 2000) {
+            dt = dt.addYears(100);
+        }
+        dt.setTimeSpec(Qt::UTC);
+        return dt;
+    }
+
+    // RCT2
+    if (const auto  rct2 = rct2Ticket(); rct2.isValid()) {
+        const auto dt = rct2.firstDayOfValidity();
+        if (dt.month() != 1 || dt.day() != 1 || !rct2.outboundDepartureStation().isEmpty()) {
+            return QDateTime(dt, {0, 0, 0});
+        }
+        // firstDayOfValidity is just a year, and we have wildcard station names
+        const auto dep = rct2.outboundDepartureTime();
+        return dep.isValid() ? dep : QDateTime(dt, {0, 0, 0});
+    }
+
+    return {};
+}
+
+QDateTime Uic9183Parser::validUntil() const
+{
+    // DB vendor block
+    const auto b = findBlock<Vendor0080BLBlock>();
+    if (b.isValid() && b.orderBlockCount() == 1) {
+        return QDateTime(b.orderBlock(0).validTo(), {23, 59, 59});
+    }
+
+    // ÖBB vender block
+    if (const auto b = findBlock("118199"); !b.isNull()) {
+        const auto obj = QJsonDocument::fromJson(QByteArray::fromRawData(b.content(), b.contentSize())).object();
+        auto dt = QDateTime::fromString(obj.value(QLatin1String("B")).toString(), QStringLiteral("yyMMddhhmm"));
+        if (dt.date().year() < 2000) {
+            dt = dt.addYears(100);
+        }
+        dt.setTimeSpec(Qt::UTC);
+        return dt;
+    }
+
+    return {};
+}
+
 Person Uic9183Parser::person() const
 {
     // Deutsche Bahn vendor block
